@@ -24,59 +24,6 @@
 
 #include "xyzsh/xyzsh.h"
 
-BOOL cmd_jobs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-
-    char buf[BUFSIZ];
-    int i;
-    for(i=0; i<vector_count(gJobs); i++) {
-        sObject* job = vector_item(gJobs, i);
-        int size = snprintf(buf, BUFSIZ, "[%d] %s (pgrp: %d)", i+1, SJOB(job).mName, SJOB(job).mPGroup);
-        if(!fd_write(nextout, buf, size)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
-        }
-
-        if(!fd_write(nextout, "\n", 1)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
-        }
-    }
-
-    if(vector_count(gJobs) == 0) {
-        runinfo->mRCode = 1;
-    }
-    else {
-        runinfo->mRCode = 0;
-    }
-
-    return TRUE;
-}
-
-BOOL cmd_fg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    if(gAppType != kATOptC) {
-        if(runinfo->mCommand->mArgsNumRuntime >= 2) {
-            int num = atoi(runinfo->mCommand->mArgsRuntime[1]);
-            if(!forground_job(num-1)) {
-                return FALSE;
-            }
-        }
-        else {
-            if(!forground_job(0)) {
-                return FALSE;
-            }
-        }
-
-        runinfo->mRCode = 0;
-    }
-
-    return TRUE;
-}
-
 BOOL cmd_exit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -133,6 +80,48 @@ BOOL cmd_while(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_for(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mBlocksNum == 1 && command->mArgsNumRuntime >= 4) {
+        char* arg_name = command->mArgsRuntime[1];
+        char* in = command->mArgsRuntime[2];
+
+        if(strcmp(in, "in") != 0) {
+            err_msg("for command needs 'in' word at 3th argument.", runinfo->mSName, runinfo->mSLine, "for");
+            return FALSE;
+        }
+
+        runinfo->mRCode = 0;
+
+        sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+
+        int i;
+        for(i=3; i<command->mArgsNumRuntime; i++) {
+            /// condition ///
+            uobject_put(object, arg_name, STRING_NEW_GC(command->mArgsRuntime[i], TRUE));
+
+            /// inner loop ///
+            int rcode = 0;
+            if(!run(command->mBlocks[0], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+                if(rcode == RCODE_BREAK) {
+                    rcode = 0;
+                    break;
+                }
+                else {
+                    runinfo->mRCode = rcode;
+                    return FALSE;
+                }
+            }
+
+            runinfo->mRCode = rcode;
+        }
+    }
+
+    return TRUE;
+}
+
 BOOL cmd_break(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     runinfo->mRCode = RCODE_BREAK;
@@ -230,130 +219,6 @@ BOOL cmd_return(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return FALSE;
 }
 
-BOOL cmd_stackinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-
-    char buf[BUFSIZ];
-
-    int size = snprintf(buf, BUFSIZ, "slot size %d\npool size %d\nall object number %d\n", stack_slot_size(), stack_page_size(), stack_slot_size()*stack_page_size());
-
-    if(!fd_write(nextout, buf, size)) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-        return FALSE;
-    }
-
-    runinfo->mRCode = 0;
-    return TRUE;
-}
-
-BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-    char buf[BUFSIZ];
-
-    uobject_it* it = uobject_loop_begin(SFUN(runinfo->mRunningObject).mLocalObjects);
-    while(it) {
-        sObject* object2 = uobject_loop_item(it);
-        char* key = uobject_loop_key(it);
-
-        char*obj_kind[16] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
-        };
-        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
-
-        if(!fd_write(nextout, buf, size)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
-        }
-
-        it = uobject_loop_next(it);
-    }
-
-    if(uobject_count(SFUN(runinfo->mRunningObject).mLocalObjects) == 0) {
-        runinfo->mRCode = 1;
-    }
-    else {
-        runinfo->mRCode = 0;
-    }
-
-    return TRUE;
-}
-
-BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sObject* running_object = runinfo->mRunningObject;
-
-    if(running_object && (TYPE(running_object) == T_FUN || TYPE(running_object) == T_CLASS) && SFUN(running_object).mRunInfo) {
-        sRunInfo* runinfo2 = SFUN(running_object).mRunInfo;
-
-        char buf[BUFSIZ];
-        sObject* object;
-
-        char current_object[128];
-        *current_object = 0;
-        object = runinfo2->mCurrentObject;
-        while(object && object != gRootObject) {
-            strcat(current_object, SUOBJECT(object).mName);
-            strcat(current_object, "<-");
-            object = SUOBJECT(object).mParent;
-        }
-        strcat(current_object, "root");
-
-        char reciever_object[128];
-        *reciever_object = 0;
-        object = runinfo2->mRecieverObject;
-        while(object && object != gRootObject) {
-            strcat(reciever_object, SUOBJECT(object).mName);
-            strcat(reciever_object, "<-");
-            object = SUOBJECT(object).mParent;
-        }
-        strcat(reciever_object, "root");
-
-        sCommand* command = runinfo2->mCommand;
-
-        char command_name[128];
-        *command_name = 0;
-        int i = 0;
-        while(i < command->mMessagesNum) {
-            strcat(command_name, command->mMessages[i++]);
-            strcat(command_name, "->");
-        }
-        strcat(command_name, command->mArgs[0]);
-
-        int size = snprintf(buf, BUFSIZ, "source name: %s\nsource line: %d\nrun nest level: %d\ncurrent object: %s\nreciever object: %s\ncommand name: %s\n", runinfo2->mSName, runinfo2->mSLine, runinfo2->mNestLevel, current_object, reciever_object, command_name);
-
-        if(!fd_write(nextout, buf, size)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
-        }
-
-        runinfo->mRCode = 0;
-    }
-
-    return TRUE;
-}
-
-BOOL cmd_gcinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-    char buf[BUFSIZ];
-
-    int size = snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object number %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
-
-    if(!fd_write(nextout, buf, size)) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-        return FALSE;
-    }
-
-    runinfo->mRCode = 0;
-    return TRUE;
-}
-
 BOOL cmd_subshell(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -370,44 +235,6 @@ BOOL cmd_subshell(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
-BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-
-    if(command->mArgsNumRuntime == 1) {
-        int num = gc_sweep();
-
-        char buf[BUFSIZ];
-        int size = snprintf(buf, BUFSIZ, "%d objects deleted\n", num);
-
-        if(!fd_write(nextout, buf, size)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
-        }
-        runinfo->mRCode = 0;
-    }
-    else {
-        int i;
-        for(i=1; i<command->mArgsNumRuntime; i++) {
-            char* var_name = command->mArgsRuntime[i];
-            sObject* current_object = runinfo->mCurrentObject;
-
-            sObject* item = uobject_item(current_object, var_name);
-
-            if(item && item->mUserObject) {
-                if(item != gCurrentObject || strcmp(var_name, SUOBJECT(item).mName) != 0) {
-                    if(uobject_erase(current_object, var_name)) {
-                        runinfo->mRCode = 0;
-                    }
-                }
-            }
-        }
-    }
-
-    return TRUE;
-}
-
 BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -469,6 +296,101 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mBlocksNum == 2) {
+        int rcode = 0;
+        if(!run(command->mBlocks[0], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+            if(rcode == RCODE_NFUN_FALSE || rcode == RCODE_BREAK || rcode == RCODE_RETURN || rcode == RCODE_EXIT) {
+                runinfo->mRCode = rcode;
+                return FALSE;
+            }
+            else {
+                if(!run(command->mBlocks[1], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+                    runinfo->mRCode = rcode;
+                    return FALSE;
+                }
+                else {
+                    runinfo->mRCode = rcode;
+                    return TRUE;
+                }
+            }
+        }
+        else {
+            runinfo->mRCode = rcode;
+            return TRUE;
+        }
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+    if(!fd_write(nextout, string_c_str(gErrMsg), string_length(gErrMsg))) {
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+        return FALSE;
+    }
+    runinfo->mRCode = 0;
+
+    return TRUE;
+}
+
+BOOL cmd_raise(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime == 2) {
+        err_msg(command->mArgsRuntime[1], runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+
+BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime == 1) {
+        int num = gc_sweep();
+
+        char buf[BUFSIZ];
+        int size = snprintf(buf, BUFSIZ, "%d objects deleted\n", num);
+
+        if(!fd_write(nextout, buf, size)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+        runinfo->mRCode = 0;
+    }
+    else {
+        int i;
+        for(i=1; i<command->mArgsNumRuntime; i++) {
+            char* var_name = command->mArgsRuntime[i];
+            sObject* current_object = runinfo->mCurrentObject;
+
+            sObject* item = uobject_item(current_object, var_name);
+
+            if(item && item->mUserObject) {
+                if(item != gCurrentObject || strcmp(var_name, SUOBJECT(item).mName) != 0) {
+                    if(uobject_erase(current_object, var_name)) {
+                        runinfo->mRCode = 0;
+                    }
+                }
+            }
+        }
+    }
+
+    return TRUE;
+}
+
 BOOL cmd_load(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -594,18 +516,129 @@ BOOL cmd_msleep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
-BOOL cmd_raise(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+BOOL cmd_stackinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
 
-    if(command->mArgsNumRuntime == 2) {
-        err_msg(command->mArgsRuntime[1], runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+    char buf[BUFSIZ];
+
+    int size = snprintf(buf, BUFSIZ, "slot size %d\npool size %d\nall object number %d\n", stack_slot_size(), stack_page_size(), stack_slot_size()*stack_page_size());
+
+    if(!fd_write(nextout, buf, size)) {
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
 
+    runinfo->mRCode = 0;
+    return TRUE;
+}
+
+BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+    char buf[BUFSIZ];
+
+    uobject_it* it = uobject_loop_begin(SFUN(runinfo->mRunningObject).mLocalObjects);
+    while(it) {
+        sObject* object2 = uobject_loop_item(it);
+        char* key = uobject_loop_key(it);
+
+        char*obj_kind[16] = {
+            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
+        };
+        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
+
+        if(!fd_write(nextout, buf, size)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+
+        it = uobject_loop_next(it);
+    }
+
+    if(uobject_count(SFUN(runinfo->mRunningObject).mLocalObjects) == 0) {
+        runinfo->mRCode = 1;
+    }
+    else {
+        runinfo->mRCode = 0;
+    }
+
     return TRUE;
 }
 
+BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sObject* running_object = runinfo->mRunningObject;
+
+    if(running_object && (TYPE(running_object) == T_FUN || TYPE(running_object) == T_CLASS) && SFUN(running_object).mRunInfo) {
+        sRunInfo* runinfo2 = SFUN(running_object).mRunInfo;
+
+        char buf[BUFSIZ];
+        sObject* object;
+
+        char current_object[128];
+        *current_object = 0;
+        object = runinfo2->mCurrentObject;
+        while(object && object != gRootObject) {
+            strcat(current_object, SUOBJECT(object).mName);
+            strcat(current_object, "<-");
+            object = SUOBJECT(object).mParent;
+        }
+        strcat(current_object, "root");
+
+        char reciever_object[128];
+        *reciever_object = 0;
+        object = runinfo2->mRecieverObject;
+        while(object && object != gRootObject) {
+            strcat(reciever_object, SUOBJECT(object).mName);
+            strcat(reciever_object, "<-");
+            object = SUOBJECT(object).mParent;
+        }
+        strcat(reciever_object, "root");
+
+        sCommand* command = runinfo2->mCommand;
+
+        char command_name[128];
+        *command_name = 0;
+        int i = 0;
+        while(i < command->mMessagesNum) {
+            strcat(command_name, command->mMessages[i++]);
+            strcat(command_name, "->");
+        }
+        strcat(command_name, command->mArgs[0]);
+
+        int size = snprintf(buf, BUFSIZ, "source name: %s\nsource line: %d\nrun nest level: %d\ncurrent object: %s\nreciever object: %s\ncommand name: %s\n", runinfo2->mSName, runinfo2->mSLine, runinfo2->mNestLevel, current_object, reciever_object, command_name);
+
+        if(!fd_write(nextout, buf, size)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_gcinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+    char buf[BUFSIZ];
+
+    int size = snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object number %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
+
+    if(!fd_write(nextout, buf, size)) {
+        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+        return FALSE;
+    }
+
+    runinfo->mRCode = 0;
+    return TRUE;
+}
 
 BOOL cmd_time(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
@@ -774,49 +807,6 @@ BOOL cmd_rehash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
-BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-
-    if(command->mBlocksNum == 2) {
-        int rcode = 0;
-        if(!run(command->mBlocks[0], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
-            if(rcode == RCODE_NFUN_FALSE || rcode == RCODE_BREAK || rcode == RCODE_RETURN || rcode == RCODE_EXIT) {
-                runinfo->mRCode = rcode;
-                return FALSE;
-            }
-            else {
-                if(!run(command->mBlocks[1], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
-                    runinfo->mRCode = rcode;
-                    return FALSE;
-                }
-                else {
-                    runinfo->mRCode = rcode;
-                    return TRUE;
-                }
-            }
-        }
-        else {
-            runinfo->mRCode = rcode;
-            return TRUE;
-        }
-    }
-
-    return TRUE;
-}
-
-BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-    if(!fd_write(nextout, string_c_str(gErrMsg), string_length(gErrMsg))) {
-        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-        return FALSE;
-    }
-    runinfo->mRCode = 0;
-
-    return TRUE;
-}
 
 sObject* gPrompt;
 
@@ -979,3 +969,56 @@ BOOL cmd_kanjicode(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     return TRUE;
 }
+
+BOOL cmd_jobs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    char buf[BUFSIZ];
+    int i;
+    for(i=0; i<vector_count(gJobs); i++) {
+        sObject* job = vector_item(gJobs, i);
+        int size = snprintf(buf, BUFSIZ, "[%d] %s (pgrp: %d)", i+1, SJOB(job).mName, SJOB(job).mPGroup);
+        if(!fd_write(nextout, buf, size)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+
+        if(!fd_write(nextout, "\n", 1)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+    }
+
+    if(vector_count(gJobs) == 0) {
+        runinfo->mRCode = 1;
+    }
+    else {
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_fg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    if(gAppType != kATOptC) {
+        if(runinfo->mCommand->mArgsNumRuntime >= 2) {
+            int num = atoi(runinfo->mCommand->mArgsRuntime[1]);
+            if(!forground_job(num-1)) {
+                return FALSE;
+            }
+        }
+        else {
+            if(!forground_job(0)) {
+                return FALSE;
+            }
+        }
+
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
