@@ -1,5 +1,6 @@
 #include "config.h"
 #include "xyzsh/xyzsh.h"
+
 #include <string.h>
 #include <strings.h>
 #include <stdlib.h>
@@ -312,11 +313,9 @@ static char* redirect_completion(const char* text, int stat)
     return NULL;
 }
 
-/*
 #if !HAVE_DECL_ENVIRON
             extern char **environ;
 #endif
-*/
 
 static char* env_completion(const char* text, int stat_)
 {
@@ -326,6 +325,30 @@ static char* env_completion(const char* text, int stat_)
         gCompletionArray = VECTOR_NEW_STACK(16);
         sObject* hash = HASH_NEW_STACK(16);
 
+        char** p;
+        for(p = environ; *p; p++) {
+            char env_name[PATH_MAX];
+
+            char* p2 = env_name;
+            char* p3 = *p;
+
+            while(*p3 != 0 && *p3 != '=') {
+                *p2++ = *p3++;
+            }
+
+            char* env = getenv(*p);
+            struct stat estat;
+            if(stat(env, &estat) >= 0) {
+                if(S_ISDIR(estat.st_mode)) {
+                    *p2++ = '/';
+                }
+            }
+            *p2 = 0;
+
+            sObject* string = STRING_NEW_STACK(env_name);
+            vector_add(gCompletionArray, string_c_str(string));
+        }
+
         sObject* current = gReadlineCurrentObject;
         while(1) {
             uobject_it* it = uobject_loop_begin(current);
@@ -350,31 +373,6 @@ static char* env_completion(const char* text, int stat_)
             if(current == gRootObject) break;
 
             current = SUOBJECT(current).mParent;
-
-/*
-            char** p;
-            for(p = environ; *p; p++) {
-                char env_name[PATH_MAX];
-
-                char* p2 = env_name;
-                char* p3 = *p;
-
-                while(*p3 != '=') {
-                    *p2++ = *p3++;
-                }
-
-                char* env = getenv(*p);
-                struct stat estat;
-                if(stat(env, &estat) >= 0) {
-                    if(S_ISDIR(estat.st_mode)) {
-                        *p2++ = '/';
-                    }
-                }
-                *p2 = 0;
-
-                vector_add(gCompletionArray, STRING_NEW_STACK(env_name));
-            }
-*/
         }
 
         vector_sort(gCompletionArray, name_sort);
@@ -1213,7 +1211,7 @@ void xyzsh_readline_init(BOOL runtime_script)
     rl_attempted_completion_function = readline_on_complete;
 //    rl_completion_entry_function = readline_filename_completion_null_generator;
     rl_completer_quote_characters = "\"'";
-    rl_completer_word_break_characters = " \t\n\"'|!&;()$%<>";
+    rl_completer_word_break_characters = " \t\n\"'|!&;()$%<>=";
     rl_completion_append_character= ' ';
     rl_filename_quote_characters = " \t\n\"'|!&;()$%<>:";
     rl_filename_quoting_function = bash_quote_filename;
