@@ -24,7 +24,20 @@ sObject* gStdout;
 sObject* gStderr;
 
 sObject* gGlobalPipe;
-//sObject* gPipes;
+
+sObject* gObjectsInPipe;
+
+BOOL add_object_to_objects_in_pipe(sObject* object, sRunInfo* runinfo, sCommand* command)
+{
+    vector_add(gObjectsInPipe, object);
+
+    if(vector_count(gObjectsInPipe) > XYZSH_OBJECTS_IN_PIPE_MAX) {
+        err_msg("overflow objects in pipe", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        return FALSE;
+    }
+
+    return TRUE;
+}
 
 enum eLineField gLineField;
 enum eKanjiCode gKanjiCode;
@@ -35,7 +48,6 @@ sObject* gJobs;
 
 void run_init(enum eAppType app_type)
 {
-    //gPipes = LIST_NEW_STACK();
     gAppType = app_type;
     gStdin = FD_NEW_STACK(kFDKindBuf, 0);
     gStdout = FD_NEW_STACK(kFDKindBuf, 0);
@@ -49,6 +61,9 @@ void run_init(enum eAppType app_type)
     gKanjiCode = kByte;
 
     gGlobalPipe = FD_NEW_STACK(kFDKindBuf, 0);
+
+    gObjectsInPipe = VECTOR_NEW_GC(10, FALSE);
+    uobject_put(gxyzshObject, "_objects_in_pipe", gObjectsInPipe);
 }
 
 void run_final()
@@ -73,8 +88,6 @@ sObject* fd_new_from_stack(int kind, int fd)
 
         SFD(self).fdbuf.mLines = VECTOR_NEW_MALLOC(32);
         SFD(self).fdbuf.mReadedLineNumber = 0;
-
-        //list_push_front(gPipes, self);
     }
 
     return self;
@@ -91,8 +104,6 @@ void fd_delete_stack(sObject* self)
         vector_delete_malloc(SFD(self).fdbuf.mLines);
 
         FREE(SFD(self).fdbuf.mBuf);
-
-        //list_erase(list_find(gPipes, self), gPipes);
     }
 }
 
@@ -1248,7 +1259,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     sObject* nextin;
     if(statment->mFlags & STATMENT_CONTEXTPIPE) {
         /// read stdin ///
-        if(pipein == gStdin && SFD(gStdin).fdbuf.mBufLen == 0) {
+        if(pipein == gStdin && SFD(gStdin).fdbuf.mBufLen == 0 && !isatty(0)) {
             if(!stdin_read_out()) {
                 sCommand* command = runinfo->mCommand;
                 if(command) {
@@ -1326,7 +1337,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         runinfo->mFilter = i != 0 || (statment->mFlags & (STATMENT_CONTEXTPIPE|STATMENT_GLOBALPIPEIN));
 
         /// expand env ///
-        if(!sCommand_expand_env_redirect(command, nextin, runinfo))
+        if(!sCommand_expand_env_redirect(command, pipein, runinfo))
         {
             return FALSE;
         }
@@ -1344,6 +1355,9 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     fd_clear(gGlobalPipe);
                     nextout = gGlobalPipe;
                 }
+                else if(statment->mFlags & STATMENT_GLOBALPIPEAPPEND) {
+                    nextout = gGlobalPipe;
+                }
                 else {
                     nextout = pipeout;
                 }
@@ -1419,7 +1433,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     }
 
                     /// expand env ///
-                    if(!sCommand_expand_env(command, object, nextin, nextout, runinfo))
+                    if(!sCommand_expand_env(command, object, pipein, nextout, runinfo))
                     {
                         return FALSE;
                     }
@@ -1486,7 +1500,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 
                 if(object) {
                     /// expand env ///
-                    if(!sCommand_expand_env(command, object, nextin, nextout, runinfo))
+                    if(!sCommand_expand_env(command, object, pipein, nextout, runinfo))
                     {
                         return FALSE;
                     }
@@ -1498,7 +1512,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                 }
                 else {
                     /// expand env ///
-                    if(!sCommand_expand_env(command, NULL, nextin, nextout, runinfo))
+                    if(!sCommand_expand_env(command, NULL, pipein, nextout, runinfo))
                     {
                         return FALSE;
                     }
@@ -1532,6 +1546,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 
 BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject* current_object, sObject* running_object)
 {
+    static unsigned int n = 0;
     static int nest_level = 0;
     nest_level++;
     if(nest_level == 50) {
@@ -1543,6 +1558,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
             err_msg("stack over flow", "xyzsh", 0, "system");
         }
         nest_level--;
+
         return FALSE;
     }
 
@@ -1572,6 +1588,12 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
             stack_end_stack();
             nest_level--;
 
+            if(nest_level == 0) {
+                vector_clear(gObjectsInPipe);
+            }
+
+            if(!(n++ % 32)) (void)gc_sweep();
+
             return FALSE;
         }
 
@@ -1586,6 +1608,10 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
+                if(nest_level == 0) {
+                    vector_clear(gObjectsInPipe);
+                }
+                if(!(n++ % 32)) (void)gc_sweep();
                 return FALSE;
             }
         }
@@ -1600,6 +1626,10 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
+                if(nest_level == 0) {
+                    vector_clear(gObjectsInPipe);
+                }
+                if(!(n++ % 32)) (void)gc_sweep();
                 return FALSE;
             }
         }
@@ -1613,6 +1643,10 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
             nest_level--;
             fd_clear(gStdout);
             fd_clear(gStderr);
+            if(nest_level == 0) {
+                vector_clear(gObjectsInPipe);
+            }
+            if(!(n++ % 32)) (void)gc_sweep();
             return FALSE;
         }
         else if(runinfo.mRCode == 128+SIGINT || gxyzshSigInt) {
@@ -1624,6 +1658,10 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
             nest_level--;
             fd_clear(gStdout);
             fd_clear(gStderr);
+            if(nest_level == 0) {
+                vector_clear(gObjectsInPipe);
+            }
+            if(!(n++ % 32)) (void)gc_sweep();
             return FALSE;
         }
         else if((statment->mFlags & STATMENT_OROR) && runinfo.mRCode == 0) {
@@ -1649,8 +1687,10 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
     stack_end_stack();
     nest_level--;
 
-static unsigned int n = 0;
-if(!(n++ % 32)) (void)gc_sweep();
+    if(nest_level == 0) {
+        vector_clear(gObjectsInPipe);
+    }
+    if(!(n++ % 32)) (void)gc_sweep();
 
     return TRUE;
 }
